//  gmodconfig - a Gnome based kernel module configuration tool
//  http://gmodconfig.sourceforge.net/
//  Copyright (C) 2003 Cyril Bortolato
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//  control.cpp: Bonobo control
//--------------------------------------------------------------------------//

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "control.h"
#include "moduletypes.h"
#include "modgui.h"
#include "modconfmgr.h"
#include "modinforeader.h"
#include "dkmsinstallmgr.h"
#include <gnome.h>
#include <iostream>
#include <stdexcept>

#include <bonobo-activation/bonobo-activation-shlib.h>
#include <bonobo/bonobo-shlib-factory.h>

using namespace gmodcfg;
using namespace std;

// Control properties

enum { PROP_SETMODCONF,   // to set the path to modules.conf
       PROP_SETLANGUAGE,  // to set the language to use
       PROP_SETVERBOSITY, // to set verbosity
       PROP_SHOWMODULE,   // display a module
       PROP_GETMODUSAGE,  // to get a module's short description
       PROP_HASUNSAVEDCHANGES, // to know whether there are unsaved changes
       PROP_INSTALLPACKAGE, // to install a module package
       PROP_UNINSTALLPACKAGE, // to un-install a module package
       PROP_SAVEOPTIONS,     // to save changes to modules.conf
};

static const char* oafId = "OAFIID:GNOME_Linux_ModuleManager_Factory";

//--------------------------------------------------------------------------//
//  Factory and activation code
//--------------------------------------------------------------------------//
static BonoboObject*
createModuleManager( BonoboGenericFactory*, const char*, gpointer )
{
    Control* ctrl = new Control();
    return BONOBO_OBJECT( ctrl->GetBonoboControl() );
}

//BONOBO_ACTIVATION_SHLIB_FACTORY( "OAFIID:GNOME_Linux_ModuleManager_Factory", "gmodconfig factory", createModuleManager, NULL )

// KLUDGE: had to expand the BONOBO_ACTIVATION_SHLIB_FACTORY macro and
// remove the "const" qualifier from Bonobo_Plugin_info definition
// else this symbol would not end up in the shared library.

static Bonobo_Unknown
make_factory( PortableServer_POA poa, const char*, gpointer impl_ptr,
              CORBA_Environment *ev)
{
    return bonobo_shlib_factory_std( oafId, poa, impl_ptr, createModuleManager,
                                     NULL, ev );
}

static BonoboActivationPluginObject plugin_list[] = {{oafId, make_factory}, { NULL } };

BonoboActivationPlugin Bonobo_Plugin_info = { plugin_list, "gmodconfig factory" };

// KLUDGE: default BONOBO_ARG_SET_STRING breaks with g++
#define MY_BONOBO_ARG_SET_STRING(a,v)  (CORBA_free (*(char **)a->_value),\
			     *((CORBA_char **)(a->_value)) = CORBA_string_dup ((v)?(v):""))

// KLUDGE: default BONOBO_ARG_GET_STRING breaks with g++
#define MY_BONOBO_ARG_GET_STRING(a)    (*((CORBA_char **)(a->_value)))

//--------------------------------------------------------------------------//
//  Get/Set properties on this control
//--------------------------------------------------------------------------//
static void
getProp( BonoboPropertyBag* pb, BonoboArg* arg, guint propId,
         CORBA_Environment* ev, gpointer data )
{
    Control* ctrl = static_cast<Control*>(data);

    switch( propId )
    {
        case PROP_SETMODCONF:
        case PROP_SETLANGUAGE:
        case PROP_SETVERBOSITY:
        case PROP_SHOWMODULE:
            break; // ignore, not supposed to be set by client

        case PROP_GETMODUSAGE:
        {
            const char* usage = ctrl->GetModuleUsage().c_str();
            MY_BONOBO_ARG_SET_STRING( arg, usage );
            break;
        }

        case PROP_HASUNSAVEDCHANGES:
        {
            bool changes = ctrl->HasUnsavedChanges();
            BONOBO_ARG_SET_BOOLEAN( arg, changes );
            break;
        }

        case PROP_INSTALLPACKAGE:
        case PROP_UNINSTALLPACKAGE:
        case PROP_SAVEOPTIONS:
            break; // ignore, not supposed to be set by client

        default:
            bonobo_exception_set( ev, ex_Bonobo_PropertyBag_NotFound );
            break;
    }
}

static void
setProp( BonoboPropertyBag* pb, const BonoboArg* arg, guint propId,
         CORBA_Environment* ev, gpointer data )
{
    Control* ctrl = static_cast<Control*>(data);

    switch( propId )
    {
        case PROP_SETMODCONF:
            ctrl->SetModulesConfFile( MY_BONOBO_ARG_GET_STRING( arg ) );
            break;

        case PROP_SETLANGUAGE:
            ctrl->SetLanguage( MY_BONOBO_ARG_GET_STRING( arg ) );
            break;

        case PROP_SETVERBOSITY:
            ctrl->SetVerbosity( BONOBO_ARG_GET_INT( arg ) );
            break;

        case PROP_SHOWMODULE:
            ctrl->ShowModule( MY_BONOBO_ARG_GET_STRING( arg ) );
            break;

        case PROP_GETMODUSAGE:
        case PROP_HASUNSAVEDCHANGES:
            break; // meaningless (read-only properties)

        case PROP_INSTALLPACKAGE:
            ctrl->InstallPackage( MY_BONOBO_ARG_GET_STRING( arg ) );
            break;

        case PROP_UNINSTALLPACKAGE:
            ctrl->UnInstallPackage( MY_BONOBO_ARG_GET_STRING( arg ) );
            break;

        case PROP_SAVEOPTIONS:
            ctrl->SaveOptions();
            break;

        default:
            bonobo_exception_set( ev, ex_Bonobo_PropertyBag_NotFound );
            break;
    }
}

//--------------------------------------------------------------------------//
//  Define properties for this control
//--------------------------------------------------------------------------//
void
Control::defineProps()
{
    BonoboPropertyBag* pb = bonobo_property_bag_new( getProp, setProp, this );
    bonobo_control_set_properties( BonoboCtrl, BONOBO_OBJREF(pb), NULL );

    bonobo_property_bag_add( pb, "setModulesConfFile", PROP_SETMODCONF,
                             BONOBO_ARG_STRING, NULL, "", 0 );

    bonobo_property_bag_add( pb, "setLanguage", PROP_SETLANGUAGE,
                             BONOBO_ARG_STRING, NULL, "", 0 );

    bonobo_property_bag_add( pb, "setVerbosity", PROP_SETVERBOSITY,
                             BONOBO_ARG_INT, 0, NULL, 0 );

    bonobo_property_bag_add( pb, "showModule", PROP_SHOWMODULE,
                             BONOBO_ARG_STRING, NULL,
                             _("Name of the module to display"), 0 );

    bonobo_property_bag_add( pb, "getModuleUsage", PROP_GETMODUSAGE,
                             BONOBO_ARG_STRING, NULL,
                             _("Short description of current module"), 0 );

    bonobo_property_bag_add( pb, "hasUnsavedChanges", PROP_HASUNSAVEDCHANGES,
                             BONOBO_ARG_BOOLEAN, NULL,
                             _("Whether there are unsaved changes"), 0 );

    bonobo_property_bag_add( pb, "installPackage", PROP_INSTALLPACKAGE,
                             BONOBO_ARG_STRING, NULL,
                             _("Install a package"), 0 );

    bonobo_property_bag_add( pb, "unInstallPackage", PROP_UNINSTALLPACKAGE,
                             BONOBO_ARG_STRING, NULL,
                             _("Un-install a package"), 0 );

    bonobo_property_bag_add( pb, "saveOptions", PROP_SAVEOPTIONS,
                             BONOBO_ARG_NULL, NULL,
                             _("Short description of current module"), 0 );

    bonobo_object_unref( BONOBO_OBJECT( pb ) );
}

//--------------------------------------------------------------------------//
//  Notify listeners of a new message
//--------------------------------------------------------------------------//
void
Control::postMessage( const string& msg )
{
    BonoboArg* arg = bonobo_arg_new( BONOBO_ARG_STRING );
    MY_BONOBO_ARG_SET_STRING( arg, msg.c_str() );
    bonobo_event_source_notify_listeners( EventSource, "message", arg, NULL );
    bonobo_arg_release( arg );
}

//--------------------------------------------------------------------------//
//  C'tor and d'tor
//--------------------------------------------------------------------------//
Control::Control() : ModInfoReader(NULL), ModConfMgr(NULL), InstallMgr(NULL),
                     Verbosity(0), InitializedBackEnd(false),
                     ModulesConfFile("/etc/modules.conf")
{
    // Use the current language by default
    SetLanguage( pango_language_to_string( gtk_get_default_language() ) );

    // Create widget
    TopWidget = gtk_vbox_new( FALSE, 0 );
    InstallMgr = new DkmsInstallManager();
    ModGuiMgr = new ModuleGuiBuilder( GTK_BOX(TopWidget), InstallMgr );
    makeButtons( GTK_BOX(TopWidget) );
    gtk_widget_show( TopWidget );

    // Create a Bonobo control for this widget and define its properties
    BonoboCtrl = bonobo_control_new( TopWidget );
    defineProps();

    // Create an event source (to post messages) and attach it to the control
    EventSource = bonobo_event_source_new();
    bonobo_object_add_interface( BONOBO_OBJECT(BonoboCtrl),
                                 BONOBO_OBJECT(EventSource) );
}

Control::~Control()
{
    delete ModInfoReader;
    delete ModConfMgr;
    delete ModGuiMgr;
    delete InstallMgr;
}

//--------------------------------------------------------------------------//
//  "Apply", "Save", "Cancel" buttons' click handlers
//--------------------------------------------------------------------------//
static void
on_apply_clicked( GtkButton* button, gpointer data )
{
    Control* ctrl = static_cast<Control*>( data );
    ctrl->ApplyOptions();
}

static void
on_save_clicked( GtkButton* button, gpointer data )
{
    Control* ctrl = static_cast<Control*>( data );
    ctrl->SaveOptions();
}

static void
on_undo_clicked( GtkButton* button, gpointer data )
{
    Control* ctrl = static_cast<Control*>( data );
    ctrl->DiscardChanges();
}

//--------------------------------------------------------------------------//
//  Build Apply, Save and Undo buttons
//--------------------------------------------------------------------------//
GtkWidget*
Control::makeButtons( GtkBox* vbox )
{
    GtkTooltips* tooltips = gtk_tooltips_new();
    GtkWidget* buttonbox = gtk_hbutton_box_new();
    gtk_box_pack_start( GTK_BOX( vbox ), buttonbox, FALSE, TRUE, 5 );
    gtk_container_set_border_width( GTK_CONTAINER( buttonbox ), 5 );
    gtk_button_box_set_layout( GTK_BUTTON_BOX( buttonbox ),
                               GTK_BUTTONBOX_START );
    gtk_button_box_set_spacing( GTK_BUTTON_BOX( buttonbox ), 5 );

    // "apply" button
    GtkWidget* apply = gtk_button_new_from_stock( "gtk-apply" );
    gtk_container_add( GTK_CONTAINER( buttonbox ), apply );
    gtk_widget_set_sensitive( apply, FALSE );
    GTK_WIDGET_SET_FLAGS( apply, GTK_CAN_DEFAULT );
    gtk_tooltips_set_tip( tooltips, apply,
        _("Load (or reload) the module with the new settings"), NULL );
    gtk_signal_connect( GTK_OBJECT( apply ), "clicked",
                        GTK_SIGNAL_FUNC( on_apply_clicked ), this );

    // "save" button
    GtkWidget* save = gtk_button_new_from_stock( "gtk-save" );
    gtk_container_add( GTK_CONTAINER( buttonbox ), save );
    GTK_WIDGET_SET_FLAGS( save, GTK_CAN_DEFAULT );
    gtk_tooltips_set_tip( tooltips, save,
        _("Save all changes made so far into the modules.conf file"), NULL );
    gtk_signal_connect( GTK_OBJECT( save ), "clicked",
                        GTK_SIGNAL_FUNC( on_save_clicked ), this );

    // "undo" button
    GtkWidget* undo = gtk_button_new_from_stock( "gtk-undo" );
    gtk_container_add( GTK_CONTAINER( buttonbox ), undo );
    GTK_WIDGET_SET_FLAGS( undo, GTK_CAN_DEFAULT );
    gtk_tooltips_set_tip( tooltips, undo,
        _("Undo changes to the current module"), NULL );
    gtk_signal_connect( GTK_OBJECT( undo ), "clicked",
                        GTK_SIGNAL_FUNC( on_undo_clicked ), this );

    gtk_widget_show_all( buttonbox );
}

//--------------------------------------------------------------------------//
//  Init internal objects if not done already
//--------------------------------------------------------------------------//
bool
Control::initBackEnd()
{
    if ( InitializedBackEnd )
        return true;

    ModConfMgr = new ModulesConfManager( ModulesConfFile, Verbosity );

    // Read the contents of the modules.conf file
    if ( !ModConfMgr->ReadModulesConf() )
    {
        gchar* msg = g_strdup_printf(
             _("Could not read the contents of %s"), ModulesConfFile.c_str() );
        postMessage( msg );
        g_free( msg );

        delete ModConfMgr;
        return false;
    }

    ModInfoReader = new ModuleInfoReader( Verbosity );
    InitializedBackEnd = true;
    return true;
}

//--------------------------------------------------------------------------//
//  Set the language we should use for the XML files
//--------------------------------------------------------------------------//
void
Control::SetLanguage( const string& str )
{
    // Get the first 2 letters of the ISO code
    Lang = str.substr(0, 2);

    if ( Verbosity )
        cout << "Using \"" << Lang << "\" language" << endl;
}

//--------------------------------------------------------------------------//
//  Populate the Parameters, etc.. tabs for a module
//  Return true for success.
//--------------------------------------------------------------------------//
bool
Control::ShowModule( const string& modName )
{
    if ( Verbosity >= 2 )
        cout << "Control: ShowModule(" << modName << ")" << endl;

    if ( !initBackEnd() )
        return false;

    const Module* mod = ModInfoReader->GetInfo( modName, Lang );
    if ( !mod )
        return false;

    // Save current module's settings
    storeCurrentModuleSettings();

    // Then move on with the new module
    ModGuiMgr->BuildGui( mod, ModConfMgr->GetOptions( mod->GetName() ) );

    return true;
}

//--------------------------------------------------------------------------//
//  Return module usage
//--------------------------------------------------------------------------//
const string&
Control::GetModuleUsage() const
{
    if ( !InitializedBackEnd )
        return EmptyString;

    const Module* mod = ModGuiMgr->GetCurrentModule();
    if ( !mod )
        return EmptyString;

    const string& usageStr = mod->GetUsage( Lang );
    if ( !usageStr.empty() )
        return usageStr;
    else
        return mod->GetUsage( "en" ); // use English as fallback
}

//--------------------------------------------------------------------------//
//  Return true if some options were changed but not saved yet
//--------------------------------------------------------------------------//
bool
Control::HasUnsavedChanges()
{
    if ( !initBackEnd() )
        return false;

    // First keep current module's settings
    storeCurrentModuleSettings();

    return ModConfMgr->HasUnsavedChanges();
}

//--------------------------------------------------------------------------//
//  Install a package located at provided URL
//--------------------------------------------------------------------------//
bool
Control::InstallPackage( const string& url )
{
    if ( Verbosity >= 2 )
        cout << "Control: InstallPackage(" << url << ")" << endl;

    if ( !initBackEnd() )
        return false;

    InstallMgr->StartInstall( url );
    return true;
}

//--------------------------------------------------------------------------//
//  Un-install a package
//--------------------------------------------------------------------------//
bool
Control::UnInstallPackage( const string& pkgName )
{
    if ( Verbosity >= 2 )
        cout << "Control: UnInstallPackage(" << pkgName << ")" << endl;

    if ( !initBackEnd() )
        return false;

    InstallMgr->StartUnInstall( pkgName );
    return true;
}

//--------------------------------------------------------------------------//
//  Load (or reload) the current module with the current (modified) options
//--------------------------------------------------------------------------//
void
Control::ApplyOptions()
{
    if ( Verbosity >= 2 )
        cout << "Control: ApplyOptions" << endl;

    if ( !initBackEnd() )
        return;

    const Module* currMod = ModGuiMgr->GetCurrentModule();
    if ( !currMod )
        return;

    // TODO
    // Check with lsmod if module's there, run rmmod, check lsmod again
    // and run modprobe
}

//--------------------------------------------------------------------------//
//  Save all the options changes into the modules.conf file
//--------------------------------------------------------------------------//
void
Control::SaveOptions()
{
    if ( Verbosity >= 2 )
        cout << "Control: SaveOptions" << endl;

    if ( !initBackEnd() )
        return;

    storeCurrentModuleSettings();

    if ( !ModConfMgr->WriteAllOptions() )
    {
        if ( Verbosity >= 3 )
            cout << "Control: SaveOptions() failed to write conf file" << endl;

        gchar* msg = g_strdup_printf(
                _("Could not save changes to %s, is the file writable?"),
                ModulesConfFile.c_str() );
        postMessage( msg );
        g_free( msg );
    }
}

//--------------------------------------------------------------------------//
//  Discard the changes made so far to the current module's parameters
//--------------------------------------------------------------------------//
void
Control::DiscardChanges()
{
    if ( Verbosity >= 2 )
        cout << "Control: DiscardChanges" << endl;

    if ( !initBackEnd() )
        return;

    const Module* currMod = ModGuiMgr->GetCurrentModule();
    if ( !currMod )
        return;

    const string& currModName = currMod->GetName();
    if ( Verbosity )
        cout << "Discarding changes for module " << currModName << endl;

    // Forget the changes and rebuild the GUI, to restore the widgets states
    ModConfMgr->SetOptions( currModName, "" );
    ModGuiMgr->BuildGui( currMod, ModConfMgr->GetOptions( currModName ) );
}

//--------------------------------------------------------------------------//
//  Store the current module's settings into the ModulesConfManager
//--------------------------------------------------------------------------//
void
Control::storeCurrentModuleSettings()
{
    const Module* currMod = ModGuiMgr->GetCurrentModule();
    if ( !currMod )
        return;

    const string& currModName = currMod->GetName();
    string options = ModGuiMgr->GenerateOptionsString();
    ModConfMgr->SetOptions( currModName, options );
    if ( Verbosity )
        cout << "Setting " << currModName << "'s options to \""
             << options << "\"" << endl;
}

