//  gmodconfig - a Gnome based kernel module configuration tool
//  http://gmodconfig.sourceforge.net/
//  Copyright (C) 2003 Cyril Bortolato
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//  control.h: Bonobo control
//--------------------------------------------------------------------------//

#ifndef control_h
#define control_h

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <string>
using std::string;

#include <bonobo.h>

namespace gmodcfg
{

class ModulesConfManager;
class ModuleGuiBuilder;
class ModuleInfoReader;
class InstallManager;

class Control
{
        BonoboControl* BonoboCtrl;
        BonoboEventSource* EventSource;

        ModuleInfoReader* ModInfoReader;
        ModulesConfManager* ModConfMgr;
        ModuleGuiBuilder* ModGuiMgr;
        InstallManager* InstallMgr;
        string ModulesConfFile, Lang, EmptyString;
        int Verbosity;
        GtkWidget* TopWidget;

        bool InitializedBackEnd;
        bool initBackEnd();
        GtkWidget* makeButtons( GtkBox* );
        void defineProps();
        void storeCurrentModuleSettings();
        void postMessage( const string& );

        Control( const Control& );
        Control& operator=( const Control& );

    public:

        Control();
        ~Control();
        BonoboControl* GetBonoboControl() const { return BonoboCtrl; }

        // Parameters: set those before using the rest of the interface
        void SetModulesConfFile( const string& f ) { ModulesConfFile = f; }
        void SetLanguage( const string& );
        void SetVerbosity( int verb ) { Verbosity = verb; }

        // Interface
        bool ShowModule( const string& name );
        const string& GetModuleUsage() const;
        bool HasUnsavedChanges();
        bool InstallPackage( const string& url );
        bool UnInstallPackage( const string& pkgName );

        // Buttons
        void ApplyOptions();
        void SaveOptions();
        void DiscardChanges();
};

};

#endif

