//  gmodconfig - a Gnome based kernel module configuration tool
//  http://gmodconfig.sourceforge.net/
//  Copyright (C) 2003 Cyril Bortolato
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//  editcallbacks.cpp: events callbacks
//--------------------------------------------------------------------------//

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <iostream>
#include <gnome.h>
#include "gmodeditapp.h"
#include "editcallbacks.h"
#include "editinterface.h"
#include "modxmlmgr.h"
#include "xsamgr.h"

using namespace gmodcfg;

extern EditorApplication* TheEditorApp;

//--------------------------------------------------------------------------//
//  gmodconfigedit's callbacks
//--------------------------------------------------------------------------//
void
on_edit_quit_activate( GtkMenuItem* menuitem, gpointer user_data )
{
    TheEditorApp->Quit();
}

void
on_edit_about_activate( GtkMenuItem* menuitem, gpointer user_data )
{
    // TODO
}

void
on_edit_xsa_newbutton_clicked( GtkButton* button, gpointer user_data )
{
    // Start the XSA druid
    TheEditorApp->NewXsa();
}

void
on_edit_xsa_openbutton_clicked( GtkButton* button, gpointer user_data )
{
    // Load the XSA file and start the druid
    TheEditorApp->LoadXsa();
}

void
on_edit_module_newbutton_clicked( GtkButton* button, gpointer user_data )
{
    // Start the module XML druid
    TheEditorApp->NewModuleInfo();
}

void
on_edit_module_openbutton_clicked( GtkButton* button, gpointer user_data )
{
    // Load the module XML file and start the druid
    TheEditorApp->LoadModuleInfo();
}

//--------------------------------------------------------------------------//
//  "XSA Assistant" druid's handlers
//--------------------------------------------------------------------------//
void
on_xsadruid_help( GnomeDruid* gnomedruid, gpointer user_data )
{
    // TODO: launch help system
}

void
on_xsadruid_cancel( GnomeDruid* gnomedruid, gpointer user_data )
{
    TheEditorApp->CloseXsaDruid();
}

//--------------------------------------------------------------------------//
//  "products" page's handlers
//--------------------------------------------------------------------------//
void
on_xsadruid_products_deletebutton_clicked( GtkButton*, gpointer )
{
    TheEditorApp->GetXsaMgr()->ProductsDeleteButtonHandler();
}

void
on_xsadruid_products_upbutton_clicked( GtkButton*, gpointer )
{
    TheEditorApp->GetXsaMgr()->ProductsUpButtonHandler();
}

void
on_xsadruid_products_downbutton_clicked( GtkButton*, gpointer )
{
    TheEditorApp->GetXsaMgr()->ProductsDownButtonHandler();
}

void
on_xsadruid_products_addbutton_clicked( GtkButton*, gpointer )
{
    TheEditorApp->GetXsaMgr()->ProductsAddButtonHandler();
}

void
on_xsadruid_products_applybutton_clicked( GtkButton*, gpointer )
{
    TheEditorApp->GetXsaMgr()->ProductsApplyButtonHandler();
}

void
on_xsadruid_products_clearbutton_clicked( GtkButton*, gpointer )
{
    TheEditorApp->GetXsaMgr()->ProductsClearButtonHandler();
}

//--------------------------------------------------------------------------//
//  XSA druid finish page's handler
//--------------------------------------------------------------------------//
void
on_xsadruid_page4_finish( GnomeDruidPage*, GtkWidget*, gpointer )
{
    TheEditorApp->GetXsaMgr()->SaveXsaFile();
    TheEditorApp->CloseXsaDruid();
}

//--------------------------------------------------------------------------//
//  "Module XML Assistant" druid's handlers
//--------------------------------------------------------------------------//
void
on_moddruid_cancel( GnomeDruid*, gpointer )
{
    TheEditorApp->CloseModXmlDruid();
}

void
on_moddruid_help( GnomeDruid* gnomedruid, gpointer user_data )
{
    // TODO: launch help system
}

//--------------------------------------------------------------------------//
//  "header" page's handlers
//--------------------------------------------------------------------------//
void
on_moddruid_header_modinfobutton_clicked( GtkButton*, gpointer )
{
    TheEditorApp->GetModXmlMgr()->HeaderModinfoButtonHandler();
}

//--------------------------------------------------------------------------//
//  "parameters" page's handlers
//--------------------------------------------------------------------------//
void
on_moddruid_param_deletebutton_clicked( GtkButton*, gpointer )
{
    TheEditorApp->GetModXmlMgr()->ParamDeleteButtonHandler();
}

void
on_moddruid_param_upbutton_clicked( GtkButton*, gpointer )
{
    TheEditorApp->GetModXmlMgr()->ParamUpButtonHandler();
}

void
on_moddruid_param_downbutton_clicked( GtkButton*, gpointer )
{
    TheEditorApp->GetModXmlMgr()->ParamDownButtonHandler();
}

void
on_moddruid_param_addbutton_clicked( GtkButton*, gpointer )
{
    TheEditorApp->GetModXmlMgr()->ParamAddButtonHandler();
}

void
on_moddruid_param_applybutton_clicked( GtkButton*, gpointer )
{
    TheEditorApp->GetModXmlMgr()->ParamApplyButtonHandler();
}

void
on_moddruid_param_clearbutton_clicked( GtkButton*, gpointer )
{
    TheEditorApp->GetModXmlMgr()->ParamClearButtonHandler();
}

void
on_moddruid_param_type_changed( GtkEditable*, gpointer )
{
    TheEditorApp->GetModXmlMgr()->ParamTypeChanged();
}

//--------------------------------------------------------------------------//
//  finish page's handler
//--------------------------------------------------------------------------//
void
on_moddruid_finish_page_finish( GnomeDruidPage*, GtkWidget*, gpointer )
{
    TheEditorApp->GetModXmlMgr()->SaveXmlFile();
    TheEditorApp->CloseModXmlDruid();
}

