//  gmodconfig - a Gnome based kernel module configuration tool
//  http://gmodconfig.sourceforge.net/
//  Copyright (C) 2003 Cyril Bortolato
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//  editmain.cpp: gmodconfig-edit's main()
//--------------------------------------------------------------------------//

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <libgnomevfs/gnome-vfs.h>

#include <iostream>
using std::cout;
using std::endl;

#include "gmodeditapp.h"
#include "editinterface.h"

using namespace gmodcfg;

EditorApplication* TheEditorApp = 0;

static int verbosity = 0;

static struct poptOption options[] =
{
    { "debug", 'd', POPT_ARG_INT, &verbosity, 1,
      N_("Prints debug messages to the console"), NULL },

    { NULL, '\0', 0, NULL, 0, NULL, NULL }
};

//--------------------------------------------------------------------------//
//  Session's "die" callback
//--------------------------------------------------------------------------//
static void
session_die( GnomeClient* client, gpointer client_data )
{
    gtk_main_quit();
}

//--------------------------------------------------------------------------//
//  main()
//--------------------------------------------------------------------------//
int
main (int argc, char *argv[])
{
#ifdef ENABLE_NLS
    bindtextdomain( GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR );
    bind_textdomain_codeset( GETTEXT_PACKAGE, "UTF-8" );
    textdomain( GETTEXT_PACKAGE );
#endif

    gnome_program_init( PACKAGE, VERSION,
                        LIBGNOMEUI_MODULE, argc, argv,
                        GNOME_PARAM_POPT_TABLE, options,
                        GNOME_PARAM_HUMAN_READABLE_NAME, "gmodconfig-edit",
                        GNOME_PARAM_APP_DATADIR, PACKAGE_DATA_DIR, NULL );

    gnome_vfs_init();

    // For session manager
    GnomeClient* client = gnome_master_client();
    gtk_signal_connect( GTK_OBJECT(client), "die",
                        GTK_SIGNAL_FUNC(session_die), NULL );

    // Start the application
    TheEditorApp = new EditorApplication( verbosity );

    gtk_main();

    delete TheEditorApp;
    return 0;
}

