//  gmodconfig - a Gnome based kernel module configuration tool
//  http://gmodconfig.sourceforge.net/
//  Copyright (C) 2003 Cyril Bortolato
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//  gmodeditapp.cpp: gmodconfig-edit's application
//--------------------------------------------------------------------------//

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "gmodeditapp.h"
#include "editinterface.h"
#include "support.h"
#include "xsamgr.h"
#include "modxmlmgr.h"

#include <iostream>
using std::cout;
using std::endl;

using namespace gmodcfg;

//--------------------------------------------------------------------------//
//  C'tor & d'tor
//--------------------------------------------------------------------------//
EditorApplication::EditorApplication( int verbosity )
                    : XsaMgr(0), ModXmlMgr(0), Verbosity(verbosity)
{
    TopWidget = create_editapp();
    gtk_widget_show( TopWidget );
    XsaFileEntry = GTK_ENTRY( lookup_widget(TopWidget, "edit_xsa_fileentry") );
    ModuleFileEntry = GTK_ENTRY( lookup_widget(TopWidget,
                                               "edit_module_fileentry") );

    // delete_event happens when the user closes the window
    gtk_signal_connect( GTK_OBJECT( TopWidget ), "delete_event",
                        GTK_SIGNAL_FUNC( gtk_main_quit ), NULL );
}

EditorApplication::~EditorApplication()
{
    delete ModXmlMgr;
    delete XsaMgr;
}

//--------------------------------------------------------------------------//
//  Quit the application
//--------------------------------------------------------------------------//
void
EditorApplication::Quit()
{
    gtk_main_quit();
}

//--------------------------------------------------------------------------//
//  Start the XSA druid
//--------------------------------------------------------------------------//
static void
xsaDeleteEvent( GtkWidget*, GdkEvent*, gpointer data )
{
    EditorApplication* app = reinterpret_cast<EditorApplication*>(data);
    app->CloseXsaDruid();
}

void
EditorApplication::startXsaMgr()
{
    XsaMgr = new XsaManager( TopWidget, Verbosity );

    GtkWidget* druidWindow = XsaMgr->GetDruidWindow();
    gtk_signal_connect( GTK_OBJECT( druidWindow ), "delete_event",
                        GTK_SIGNAL_FUNC( xsaDeleteEvent ), this );
}

//--------------------------------------------------------------------------//
//  Interface to XSA
//--------------------------------------------------------------------------//
void
EditorApplication::NewXsa()
{
    if ( XsaMgr )
        return; // already running

    const gchar* filename = gtk_entry_get_text( XsaFileEntry );
    if ( strcmp(filename, "" ) )
    {
        startXsaMgr();
        XsaMgr->StartNew( filename );
    }
}

void
EditorApplication::LoadXsa()
{
    if ( XsaMgr )
        return; // already running

    const gchar* filename = gtk_entry_get_text( XsaFileEntry );
    if ( strcmp(filename, "" ) )
    {
        startXsaMgr();
        if ( !XsaMgr->OpenXsa( filename ) )
            CloseXsaDruid();
    }
}

void
EditorApplication::CloseXsaDruid()
{
    if ( Verbosity )
        cout << "Closing XSA druid window..." << endl;

    GtkWidget* druid = XsaMgr->GetDruidWindow();
    gtk_widget_destroy( druid );

    delete XsaMgr;
    XsaMgr = 0;
}

//--------------------------------------------------------------------------//
//  Start the module XML druid
//--------------------------------------------------------------------------//
static void
modxmlDeleteEvent( GtkWidget*, GdkEvent*, gpointer data )
{
    EditorApplication* app = reinterpret_cast<EditorApplication*>(data);
    app->CloseModXmlDruid();
}

void
EditorApplication::startModXmlMgr()
{
    ModXmlMgr = new ModuleXmlManager( TopWidget, Verbosity );

    GtkWidget* druidWindow = ModXmlMgr->GetDruidWindow();
    gtk_signal_connect( GTK_OBJECT( druidWindow ), "delete_event",
                        GTK_SIGNAL_FUNC( modxmlDeleteEvent ), this );
}

//--------------------------------------------------------------------------//
//  Interface to module info
//--------------------------------------------------------------------------//
void
EditorApplication::NewModuleInfo()
{
    if ( ModXmlMgr )
        return; // already running

    const gchar* filename = gtk_entry_get_text( ModuleFileEntry );
    if ( strcmp(filename, "" ) )
    {
        startModXmlMgr();
        ModXmlMgr->StartNew( filename );
    }
}

void
EditorApplication::LoadModuleInfo()
{
    if ( ModXmlMgr )
        return; // already running

    const gchar* filename = gtk_entry_get_text( ModuleFileEntry );
    if ( strcmp(filename, "" ) )
    {
        startModXmlMgr();
        if ( !ModXmlMgr->OpenModuleXml( filename ) )
            CloseModXmlDruid();
    }
}

void
EditorApplication::CloseModXmlDruid()
{
    if ( Verbosity )
        cout << "Closing ModXml druid window..." << endl;

    GtkWidget* druid = ModXmlMgr->GetDruidWindow();
    gtk_widget_destroy( druid );

    delete ModXmlMgr;
    ModXmlMgr = 0;
}

