//  gmodconfig - a Gnome based kernel module configuration tool
//  http://gmodconfig.sourceforge.net/
//  Copyright (C) 2003 Cyril Bortolato
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//  main.cpp: main()
//--------------------------------------------------------------------------//

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <libgnomevfs/gnome-vfs.h>
#include <bonobo.h>

#include <iostream>
using std::cout;
using std::endl;

#include "gmodapp.h"
#include "interface.h"
#include <sys/utsname.h>

using namespace gmodcfg;

Application* TheApp = 0;

static int verbosity = 0;
static const gchar* modDir = 0;
static const gchar* modConfFile = "/etc/modules.conf";

static struct poptOption options[] =
{
    { "debug", 'd', POPT_ARG_INT, &verbosity, 1,
      N_("Prints debug messages to the console"), NULL },

    { "file", 'f', POPT_ARG_STRING, &modConfFile, 1,
      N_("Override path to the modules.conf file. Useful for testing :-)"),
      NULL },

    { "moduledir", 'm', POPT_ARG_STRING, &modDir, 1,
      N_("Path to the kernel modules directory"), NULL },

    { NULL, '\0', 0, NULL, 0, NULL, NULL }
};

//--------------------------------------------------------------------------//
//  Session's "die" callback
//--------------------------------------------------------------------------//
static void
session_die( GnomeClient* client, gpointer client_data )
{
    gtk_main_quit();
}

//--------------------------------------------------------------------------//
//  main()
//--------------------------------------------------------------------------//
int
main (int argc, char *argv[])
{
    gnome_program_init( PACKAGE, VERSION,
                        LIBGNOMEUI_MODULE, argc, argv,
                        GNOME_PARAM_POPT_TABLE, options,
                        GNOME_PARAM_HUMAN_READABLE_NAME, "gmodconfig",
                        GNOME_PARAM_APP_DATADIR, PACKAGE_DATA_DIR, NULL );

    gnome_vfs_init();

    // For session manager
    GnomeClient* client = gnome_master_client();
    gtk_signal_connect( GTK_OBJECT(client), "die",
                        GTK_SIGNAL_FUNC(session_die), NULL );

    // Init bonobo
    if ( !bonobo_init( &argc, argv ) )
        return 1;

    bonobo_activate();

    // Get the ISO code of the current language
    string lang = pango_language_to_string( gtk_get_default_language() );
    if ( verbosity )
        cout << "Current language is " << lang << endl;

    string str;
    if ( !modDir )
    {
        // Determine the path to the running kernel's modules
        struct utsname unameData;
        uname( &unameData );
        str = "/lib/modules/";
        str += unameData.release;
        str += "/kernel";
        modDir = str.c_str();
    }

    // Start the application
    TheApp = new Application( modDir, modConfFile, verbosity );
    TheApp->Start();

    return 0;
}

