//  gmodconfig - a Gnome based kernel module configuration tool
//  http://gmodconfig.sourceforge.net/
//  Copyright (C) 2003 Cyril Bortolato
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//  modconfmgr.h: interface to the modules.conf reader/writer
//--------------------------------------------------------------------------//

#ifndef modconfmgr_h
#define modconfmgr_h

#include <map>
#include <string>
using std::map;
using std::string;

namespace gmodcfg {

class Module;

class ModulesConfManager
{
        typedef map<string, string> StringStringMap_t;
        typedef map<string, int> StringIntMap_t;

        // key: modName, value: options
        StringStringMap_t CurrentOptionsMap; // what's in modules.conf
        StringStringMap_t ChangesMap;  // not committed changes

        StringIntMap_t CurrentOptionsLineMap;
        string ModulesConfFile;
        const string EmptyString;
        int Verbosity;

        int getOptionLine( const string& ) const;

    public:
        explicit ModulesConfManager( const string& modConfFile,
                                     int verbosity=0 );

        // Load up the contents of the modules.conf file
        bool ReadModulesConf() throw();

        // Returns the list of options associated to a module or "" if none
        const string& GetOptions( const string& ) const;

        // Set (and overwrites) options of a module. Nothing written to file.
        void SetOptions( const string&, const string& ) throw();

        // Returns whether changes to options are not saved yet
        bool HasUnsavedChanges() const;

        // Write out all the options that have been set with SetOptions
        // to the modules.conf file.
        bool WriteAllOptions();
};

};

#endif
