//  gmodconfig - a Gnome based kernel module configuration tool
//  http://gmodconfig.sourceforge.net/
//  Copyright (C) 2003 Cyril Bortolato
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//  modgui.h: build GUI that displays informations on a module
//--------------------------------------------------------------------------//

#ifndef modgui_h
#define modgui_h

#include <map>
#include <string>
using std::string;

#include <gnome.h>

namespace gmodcfg
{

class Param;
class ParameterGroup;
class Module;
class XsaTreeViewManager;
class InstallManager;

class ModuleGuiBuilder
{
        GtkNotebook* Notebook;
        GtkTooltips* Tooltips;
        const Module* CurrentModule;
        typedef std::map<const Param*, GtkWidget*> MapParamWidget;
        MapParamWidget WidgetMap;
        int Verbosity;
        XsaTreeViewManager* TreeViewMgr;
        InstallManager* InstallMgr;

        typedef std::map<string, string> StringStringMap_t;
        void parseOptions( const string&, StringStringMap_t& ) const;
        void buildParametersTab( const Module*, const StringStringMap_t& );
        void buildParameterGroup( const ParameterGroup*, GtkTable*, int,
                                  const StringStringMap_t& );
        void buildParameter( const Param*, GtkTable*, int,
                             const StringStringMap_t& );
        void buildSetupTab( const Module* );
        void buildUpdatesTab( const Module* );
        void buildAboutTab( const Module* );
        void updateNameAndUsage( const Module* );

        ModuleGuiBuilder( const ModuleGuiBuilder& );
        ModuleGuiBuilder& operator=( const ModuleGuiBuilder& );

    public:

        ModuleGuiBuilder( GtkBox* vbox, InstallManager*, int v=0 );
        ~ModuleGuiBuilder();

        // Dynamically fill in the notebook tabs with Module data, using
        // the provided options string.
        void BuildGui( const Module*, const string& );

        // Current module
        const Module* GetCurrentModule() const { return CurrentModule; }

        // Generate an option string for modules.conf
        string GenerateOptionsString() const;
};

};

#endif
