//  gmodconfig - a Gnome based kernel module configuration tool
//  http://gmodconfig.sourceforge.net/
//  Copyright (C) 2003 Cyril Bortolato
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//  modinforeader.h: interface to the module info reader
//--------------------------------------------------------------------------//

#ifndef modinforeader_h
#define modinforeader_h

#include <string>
using std::string;

#include <map>
#include <iostream>

namespace gmodcfg {

class Module;

class ModuleInfoReader
{
        int Verbosity;

        typedef std::map<string, Module*> StringModuleMap_t;
        StringModuleMap_t ModuleCache;
        Module* getModule( const string& ) const;
        void addModule( const string&, Module* );
        void replaceModule( const string&, Module* );

        ModuleInfoReader( const ModuleInfoReader& );
        ModuleInfoReader& operator=( const ModuleInfoReader& );

    public:
        explicit ModuleInfoReader( int v=0 ) : Verbosity(v) {}
        ~ModuleInfoReader();

        // Get module information, either from the cache, or from an XML
        // file or from modinfo's output
        const Module* GetInfo( const string& modName, const string& lang ) ;

        // Throw away the cache
        void ClearCache();

        // Build a Module from modinfo's output only.
        // Returns 0 if failure.
        Module* ReadFromModinfo( const string& ) const;

        // Build a Module from XML infos read from a stream
        // Returns 0 if failure.
        Module* ReadFromStream( std::istream& ) const;
};

};

#endif
