//  gmodconfig - a Gnome based kernel module configuration tool
//  http://gmodconfig.sourceforge.net/
//  Copyright (C) 2003 Cyril Bortolato
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//  pipereader.cpp: helper class to attach an ifstream to a pipe
//--------------------------------------------------------------------------//

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "pipereader.h"
#include <stdexcept>

using std::string;
using std::ifstream;

using namespace gmodcfg;

PipeReader::PipeReader( const string& cmd, ifstream& ifs ) throw()
{
    PipeFP = popen( cmd.c_str(), "r" );
    if ( !PipeFP )
        throw std::runtime_error( "Could not open pipe to" + cmd);

    // Convenient GNU extension to hook-up our C pipe to a C++ stream
    // See ext/stdio_filebuf.h for details
    PipeFileBuf = new gnufilebuf( PipeFP, std::ios_base::in );

    // Attach our filebuf to the provided ifstream
    ifs.std::basic_ios<char>::rdbuf( PipeFileBuf );
}

PipeReader::~PipeReader()
{
    pclose( PipeFP );
    delete PipeFileBuf;
}

