//  gmodconfig - a Gnome based kernel module configuration tool
//  http://gmodconfig.sourceforge.net/
//  Copyright (C) 2003 Cyril Bortolato
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//  treeviewmgr.cpp: a class to manage a GtkTreeView
//--------------------------------------------------------------------------//

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "treeviewmgr.h"

using namespace gmodcfg;

//--------------------------------------------------------------------------//
//  TreeViewManager class
//--------------------------------------------------------------------------//
TreeViewManager::TreeViewManager( GtkTreeView* treeView )
        : ListStore(0), TreeView(treeView) {}

//--------------------------------------------------------------------------//
//  Create the list store (template pattern)
//--------------------------------------------------------------------------//
void
TreeViewManager::InitListStore()
{
    createListStore();

    if ( !ListStore )
        return;

    // Associate the list store to the treeview
    GtkTreeModel* model = GTK_TREE_MODEL( ListStore );
    gtk_tree_view_set_model( TreeView, model );

    initTreeView();
}

//--------------------------------------------------------------------------//
//  Add a new entry to the treeview
//--------------------------------------------------------------------------//
void
TreeViewManager::AddEntry()
{
    GtkTreeIter iter;
    gtk_list_store_append( ListStore, &iter );
    setListEntry( &iter );
}

//--------------------------------------------------------------------------//
//  Remove the selected entry from the treeview
//--------------------------------------------------------------------------//
void
TreeViewManager::DeleteSelectedEntry()
{
    GtkTreeSelection* selection = gtk_tree_view_get_selection( TreeView );
    GtkTreeIter iter;
    if ( gtk_tree_selection_get_selected( selection, NULL, &iter ) )
        gtk_list_store_remove( ListStore, &iter );
}

//--------------------------------------------------------------------------//
//  Updates selected entry
//--------------------------------------------------------------------------//
void
TreeViewManager::UpdateSelectedEntry()
{
    GtkTreeSelection* selection = gtk_tree_view_get_selection( TreeView );
    GtkTreeIter iter;
    if ( gtk_tree_selection_get_selected( selection, NULL, &iter ) )
        setListEntry( &iter );
}

//--------------------------------------------------------------------------//
//  Move the selected entry up one notch in the treeview
//--------------------------------------------------------------------------//
void
TreeViewManager::MoveSelectedEntryUp()
{
    GtkTreeSelection* selection = gtk_tree_view_get_selection( TreeView );
    GtkTreeIter iter;
    if ( gtk_tree_selection_get_selected( selection, NULL, &iter ) )
    {
        GtkTreeModel* model = GTK_TREE_MODEL( ListStore );
        GtkTreePath* path = gtk_tree_model_get_path( model, &iter );
        if ( gtk_tree_path_prev( path ) )
        {
            GtkTreeIter oldpos = iter;
            gtk_tree_model_get_iter( model, &iter, path );

            GtkTreeIter newpos;
            gtk_list_store_insert_before( ListStore, &newpos, &iter );
            moveListItem( &oldpos, &newpos );
            gtk_tree_selection_select_iter( selection, &newpos );
        }
    }
}

//--------------------------------------------------------------------------//
//  Move the selected entry down one notch in the treeview
//--------------------------------------------------------------------------//
void
TreeViewManager::MoveSelectedEntryDown()
{
    GtkTreeSelection* selection = gtk_tree_view_get_selection( TreeView );
    GtkTreeIter iter;
    if ( gtk_tree_selection_get_selected( selection, NULL, &iter ) )
    {
        GtkTreeModel* model = GTK_TREE_MODEL( ListStore );
        GtkTreeIter oldpos = iter;
        if ( gtk_tree_model_iter_next( model, &iter ) )
        {
            GtkTreeIter newpos;
            gtk_list_store_insert_after( ListStore, &newpos, &iter );
            moveListItem( &oldpos, &newpos );
            gtk_tree_selection_select_iter( selection, &newpos );
        }
    }
}

