//  gmodconfig - a Gnome based kernel module configuration tool
//  http://gmodconfig.sourceforge.net/
//  Copyright (C) 2003 Cyril Bortolato
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//  xmlutils.cpp: helper functions for XML parsing
//--------------------------------------------------------------------------//

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "xmlutils.h"
#include <libxml/parser.h>
#include <stdexcept>

using namespace std;

//--------------------------------------------------------------------------//
//  Helper function to retrieve an attribute on a node
//--------------------------------------------------------------------------//
string
gmodcfg::XmlGetAttribute( xmlNodePtr node, const char* name )
{
    xmlChar* prop = xmlGetProp( node, (const xmlChar *)name );
    string str = prop ? (const char*)prop : "";
    xmlFree( prop );
    return str;
}

//--------------------------------------------------------------------------//
//  Helper function to retrieve a node's content
//--------------------------------------------------------------------------//
string
gmodcfg::XmlGetContent( xmlDocPtr doc, xmlNode* node )
{
    xmlChar* text = xmlNodeListGetString( doc, node->xmlChildrenNode, 1 );
    string str = text ? (const char*)text : "";
    xmlFree( text );
    return str;
}

//--------------------------------------------------------------------------//
//  Helper function to parse an input stream
//--------------------------------------------------------------------------//
xmlDoc*
gmodcfg::XmlParseFromStream( istream& is )
{
    xmlKeepBlanksDefault( 0 );
    xmlParserCtxt* ctx = xmlCreatePushParserCtxt( 0, 0, 0, 0, "" );
    if ( !ctx )
        throw std::runtime_error("Error while building the parser");

    // Read lines from input stream, one by one
    string line;
    while( getline(is, line) )
    {
        line += "\n";
        xmlParseChunk( ctx, line.c_str(), line.length(), 0 );
    }
    xmlParseChunk( ctx, 0, 0, 1 );

    // Check document is well formed and not empty
    if ( !ctx->wellFormed )
        throw std::runtime_error("Error while parsing the XML data");

    return ctx->myDoc;
}

