//  gmodconfig - a Gnome based kernel module configuration tool
//  http://gmodconfig.sourceforge.net/
//  Copyright (C) 2003 Cyril Bortolato
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//  xsagui.h: GUI utilities for XSA data
//--------------------------------------------------------------------------//

#ifndef xsagui_h
#define xsagui_h

#include <gnome.h>
#include "treeviewmgr.h"
#include <set>
#include <string>

namespace gmodcfg {

enum { XSA_PRODUCT_NAME=0, XSA_PRODUCT_VERSION, XSA_PRODUCT_LASTRELEASE,
       XSA_PRODUCT_ID, XSA_PRODUCT_URL, XSA_PRODUCT_CHANGES, XSA_PRODUCT_FILE,
       XSA_NUM_COLUMNS };

class XsaData;

class XsaTreeViewManager : public TreeViewManager
{
        const XsaData* Data;
        GtkEntry *ProdIdEntry, *ProdNameEntry, *ProdVersionEntry,
                 *ProdUrlEntry, *ProdChangesEntry, *ProdFileEntry;
        GnomeDateEdit* ProdLastRelEntry;
        std::set<std::string> Entries;

    public:
        XsaTreeViewManager( GtkTreeView* treeView, const XsaData* data )
            : TreeViewManager(treeView), ProdIdEntry(0), ProdNameEntry(0),
              ProdVersionEntry(0), ProdUrlEntry(0),
              ProdChangesEntry(0), ProdFileEntry(0), ProdLastRelEntry(0),
              Data(data) {}

        XsaTreeViewManager( GtkTreeView* treeView, GtkEntry* id,
                GtkEntry* name, GtkEntry* version, GtkEntry* url,
                GtkEntry* changes, GtkEntry* file, GnomeDateEdit* lastrel,
                const XsaData* data )
            : TreeViewManager(treeView), ProdIdEntry(id), ProdNameEntry(name),
              ProdVersionEntry(version), ProdUrlEntry(url),
              ProdChangesEntry(changes), ProdFileEntry(file),
              ProdLastRelEntry(lastrel), Data(data) {}

        // Get filename of selected entry (used by install manager)
        const gchar* GetSelectedFile() const;

        // Display details of a product
        void ShowProduct( GtkTreeIter* );

        // Insert/remove product id (to avoid duplicates in the treeview)
        bool InsertProduct( const gchar* );
        void RemoveProduct( const gchar* );

    private:
        virtual void createListStore();
        virtual void initTreeView();
        virtual void moveListItem( GtkTreeIter* from, GtkTreeIter* to );
        virtual void setListEntry( GtkTreeIter* );
};

};

#endif
