//  gmodconfig - a Gnome based kernel module configuration tool
//  http://gmodconfig.sourceforge.net/
//  Copyright (C) 2003 Cyril Bortolato
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//  xsatypes.h: XsaData class
//--------------------------------------------------------------------------//

#ifndef xsatypes_h
#define xsatypes_h

#include <string>
using std::string;

#include <vector>

namespace gmodcfg {

class XsaData
{
    public:
        XsaData() {}
        ~XsaData();

        struct Product
        {
            Product( const string& id_, const string& name_,
                     const string& ver_, const string& rel_,
                     const string& url_, const string& changes_,
                     const string& file_ )
                    : Id(id_), Name(name_), Version(ver_), LastRelease(rel_),
                      InfoUrl(url_), Changes(changes_), File(file_) {}
            string Id, Name, Version, LastRelease, InfoUrl, Changes, File;
        };

        typedef std::vector<Product*> ProductVector;

        void SetVendorName( const string& str ) { VendorName = str; }
        void SetVendorEmail( const string& str ) { VendorEmail = str; }
        void SetVendorUrl( const string& str ) { VendorUrl = str; }

        void AddProduct( const string& id_, const string& name_,
                         const string& ver_, const string& rel_,
                         const string& url_, const string& changes_,
                         const string& file_ );
        void SetProducts( const ProductVector& p ) { Products = p; }
        void ClearProducts();

        const string& GetVendorName() const { return VendorName; }
        const string& GetVendorEmail() const { return VendorEmail; }
        const string& GetVendorUrl() const { return VendorUrl; }

        ProductVector::const_iterator BeginProducts() const
                                                  { return Products.begin(); }
        ProductVector::const_iterator EndProducts() const
                                                  { return Products.end(); }
        unsigned int GetNumProducts() const { return Products.size(); }

    private:
        string VendorName, VendorEmail, VendorUrl;
        ProductVector Products;

        XsaData( const XsaData& );
        XsaData& operator=( const XsaData& );
};

};

#endif
