//  gmodconfig - a Gnome based kernel module configuration tool
//  http://gmodconfig.sourceforge.net/
//  Copyright (C) 2003 Cyril Bortolato
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//  xsawriter.cpp: write an XSA file to an ostream
//--------------------------------------------------------------------------//

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "xsawriter.h"
#include "xsatypes.h"

using std::endl;

using namespace gmodcfg;

//--------------------------------------------------------------------------//
//  Write out XSA's headers
//--------------------------------------------------------------------------//
static void
banner( ostream& os )
{
    os << "<?xml version=\"1.0\"?>" << endl;
    os << "<!DOCTYPE xsa PUBLIC \"-//LM Garshol//DTD XML Software Autoupdate 1.0//EN//XML\"" << endl;
    os << "                     \"http://www.garshol.priv.no/download/xsa/xsa.dtd\">" << endl;
    os << "<!-- generated by gmodconfig's XSA editor -->" << endl;
    os << "<!-- visit http://gmodconfig.sourceforge.net/ for informations -->" << endl;
}

//--------------------------------------------------------------------------//
//  Write out a vendor
//--------------------------------------------------------------------------//
static void
writeVendor( ostream& os, const XsaData* data )
{
    os << "<vendor>" << endl;
    os << "<name>" << data->GetVendorName() << "</name>" << endl;
    os << "<email>" << data->GetVendorEmail() << "</email>" << endl;
    os << "<url>" << data->GetVendorUrl() << "</url>" << endl;
    os << "</vendor>\n" << endl;
}

//--------------------------------------------------------------------------//
//  Write out a product entry
//--------------------------------------------------------------------------//
static void
writeProduct( ostream& os, const XsaData::Product* prod )
{
    os << "<product id=\"" << prod->Id << "\">" << endl;
    os << "<name>" << prod->Name << "</name>" << endl;
    os << "<version>" << prod->Version << "</version>" << endl;
    os << "<last-release>" << prod->LastRelease << "</last-release>" << endl;
    os << "<info-url>" << prod->InfoUrl << "</info-url>" << endl;
    os << "<changes>" << prod->Changes << "</changes>" << endl;
    os << "<file>" << prod->File << "</file>" << endl;
    os << "</product>\n" << endl;
}

//--------------------------------------------------------------------------//
//  Main entry point
//--------------------------------------------------------------------------//
void
XsaWriter::Write( const XsaData* data, ostream& os ) const
{
    banner( os );
    os << "\n<xsa>\n" << endl;
    writeVendor( os, data );

    XsaData::ProductVector::const_iterator pit = data->BeginProducts();
    XsaData::ProductVector::const_iterator pitend = data->EndProducts();
    for( ; pit != pitend; ++pit )
    {
        const XsaData::Product* prod = *pit;
        std::cout << "XsaWriter: writing " << prod->Id << endl;
        writeProduct( os, prod );
    }
    os << "</xsa>" << endl;
}

